#!/bin/bash
#    security_get.sh
# check for available updates and get/apply them
#
# 06/05/2017 acress - add SecurityAutoUpdate feature
# 06/07/2017 acress - clean up leftover files in /var/cache
# 06/27/2017 acress - do not clean up leftover files if bg mode
# 12/20/2017 acress - fix if vmware_instant_recovery returns nothing
# 02/06/2017 acress - alert if autoupdate applied
# 03/21/2017 acress - v1.6: if autoupdate, also wget latest script
ver=1.6
BPDIR=/usr/bp
dbg=0
vardir=/var/cache
upddir=/var/cache
platcfg=/var/opt/unitrends/platform_capabilities
log=/var/log/unitrends-security.log

is_activity()
{
  stats=`psql bpdb postgres -tAc "select status from bp.jobs where percent_done <> 100;"`
  nactive=0
  for s in $stats
  do
     show_status $s |head -n2 |grep -q ST_CANCELLED
     if [ $? -ne 0 ]; then
         nactive=`expr $nactive + 1`
     fi
  done
  if [ $nactive -gt 0 ]; then
     echo "There are $nactive active jobs, please wait or cancel the jobs."
     return 1
  fi

  # check if samba is in use (FLR, RDR, WinIR)
  flr=`$BPDIR/bin/cmc_vdisk_mount status |wc -l`
  if [ $flr -gt 0 ]; then
     echo "There are $flr active FLR instances, please wait or stop them."
     return 1
  fi
  IDLIST=`$BPDIR/bin/vmware_instant_recovery status | awk -F '|' '{print $1;}'`
  if [ -n "$IDLIST" -a "$IDLIST" != "none" ]; then
     echo "There are active VMIR instances, please wait or stop them."
     return 1
  fi
  IDLIST=`$BPDIR/bin/hyperv_instant_recovery status | awk -F '|' '{print $1;}'`
  if [ -n "$IDLIST" -a "$IDLIST" != "none" ]; then
     echo "There are active HVIR instances, please wait or stop them."
     return 1
  fi
  nirs=`psql bpdb postgres -tAc "select * from bp.virtual_clients where vmware_host is not NULL or hyperv_host is not NULL;" |wc -l`
  if [ $nirs -gt 0 ]; then
     echo "There are $nirs remote IR sessions, please wait or stop them."
     return 1
  fi
  #Should also check if any RDR sessions active

  return 0
}

update_available()
{
  tmpf=/tmp/wget.tmp

  cd $vardir
  rm -f security_version.txt* 
  rver0=`rpm -q unitrends-security`
  wget ftp://ftp.unitrends.com/utilities/security_version.txt >$tmpf 2>&1
  rver1=`cat security_version.txt`
  if [ $dbg -eq 1 ]; then
     echo "rver0=$rver0"
     echo "rver1=$rver1"
  fi
  cd $mydir
  if [ "$rver1" = "" ]; then
     tail -n5 $tmpf
     echo "Cannot get remote update version"
     rm -f $tmpf
     return 0
  fi
  rm -f $tmpf
  # need to tell if new version is greater or not (not equal -> greater)
  if [ "$rver0" = "$rver1" ]; then
     echo "Already have the latest update: $rver0"
     return 0
  fi
  echo "$rver1 is available" 
  return 1
}

get_vtype()
{
    vtype=rs  # Recovery series
    grep -q Metal /etc/unitrends-asset
    if [ $? -eq 0 ]; then
        vtype=metal
    fi
    lsmod |grep -q vmxnet
    if [ $? -eq 0 ]; then
        vtype=vmware
    fi
    lsmod |grep -q vmbus
    if [ $? -eq 0 ]; then
        vtype=hyperv
    fi
    ls /dev/xvd* >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        vtype=xen
    fi
    grep -q -i azure $platcfg
    if [ $? -eq 0 ]; then
        vtype=azure
    fi
    echo "vtype=$vtype" >>$log
    echo "$vtype"
}

vercmp () {
    if [[ $1 == $2 ]]
    then
        return 0
    fi
    local IFS=.
    local i ver1=($1) ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}

need_krpm () {
   uname -r |grep -q el6
   if [ $? -ne 0 ]; then
      # not el6, do nothing
      return
   fi
   rpmv=`rpm -q kernel |tail -n1`
   kv=`echo $rpmv |sed -e 's/kernel-2.6.32-//' -e 's/.el6.*//'`
   vercmp $kv 696.18.7
   if [ $? -eq 2 ]; then
		k_ok=1
		vm=`get_vtype`
		case $vm in
			azure)
				k_ok=0
				;;
			hyperv)
				k_ok=0
				;;
			xen)
				k_ok=0  # test this? grub issues?
				;;
			*)
				;;
		esac
		# if physical, VMware, show message
		if [ $k_ok -eq 1 ]; then
			echo "Please reboot after kernel update completes in about 5 minutes."
		fi
   fi
}

checkAvailableUpdate() 
{
  tmpa=/tmp/security_avail.tmp
  update_available >$tmpa
  rv=$?
  msg=`cat $tmpa`
  rm -f $tmpa
  if [ $rv -eq 0 ]; then
     # already latest
     echo "$msg"
     return 0
  fi
  update=`$BPDIR/bin/bputil -g "Configuration Options" SecurityAutoUpdate 0 $BPDIR/bpinit/master.ini`
  alert=`$BPDIR/bin/bputil -g "Configuration Options" SecurityAlert 0 $BPDIR/bpinit/master.ini`
  if [ $dbg -eq 1 ]; then
	echo "alert=$alert update=$update"
  fi
  echo "$msg"
  if [ $update -gt 0 ]; then
      # use tmpdir/security_get.sh copy because the rpm may update it
	tmpdir=/var/cache
	mydir=`pwd`
	cd $tmpdir
	wget -q ftp://ftp.unitrends.com/utilities/security_get.sh
	if [ $? -ne 0 ]; then
	   echo "wget error, use $BPDIR copy" >>$log
	   cp -f $BPDIR/bin/security_get.sh $tmpdir 2>>$log
	fi
	chmod +x $tmpdir/security_get.sh
	cd $mydir
	$tmpdir/security_get.sh apply auto
	# this would not return if the apply updated this script
  elif [ $alert -gt 0 ]; then
      # if SecurityAlert=1, send notification
      action="Go to Configure/Edit Appliance/Advanced/Support Toolbox/Security to update, or set SecurityAutoUpdate=1"
      $BPDIR/bin/send_notification.php 138 "Security Update" "$msg. $action"  >$tmpa 2>&1
      echo "Send alert (ret=$?): $msg" |tee -a $log
      rm -f $tmpa
  fi
  return 1
}

applySecurityUpdate() 
{
  tmpf=/tmp/wget.tmp
  sum0=/tmp/mdsum0.tmp
  sum1=/tmp/mdsum1.tmp
  tarball=security_updates.tar.gz
  mydir=`pwd`
  param=$1

  update_available
  if [ $? -eq 0 ]; then
     # already latest
     return 0
  fi

  is_activity |tee -a $log
  if [ $? -ne 0 ]; then
     # activity, so abort
     return 1
  fi
  echo "$parm" |grep -q "^/"
  if [ $? -eq 0 ]; then
     upddir=$parm
     echo "`date` Using $upddir for security_updates" >>$log
  fi
  cd $upddir
  if [ "$param" = "bg" ]; then
     need_krpm  |tee -a $log
  fi
  rm -rf security_updates.tar.md5* security_updates.tar.gz* updates 
  wget ftp://ftp.unitrends.com/utilities/security_updates.tar.md5 >>$tmpf 2>&1
  echo "`date` Downloading $tarball " |tee -a $log
  wget ftp://ftp.unitrends.com/utilities/$tarball >>$tmpf 2>&1 
  if [ ! -f $tarball ]; then
     tail -n10 $tmpf |tee -a $log
     echo "Error downloading $tarball" |tee -a $log
     rm -f $tmpf
     return 1
  fi
  rm -f $tmpf
  cat security_updates.tar.md5 >$sum0
  md5sum $tarball >$sum1
  s0=`awk '{ print $1 }' $sum0`
  s1=`awk '{ print $1 }' $sum1`
  rm -f $sum0 $sum1
  if [ $dbg -eq 1 ]; then
     echo "sum0=$s0"
     echo "sum1=$s1"
  fi
  if [ "$s0" != "$s1" ]; then
     echo "Error: Checksums do not match, try again later" |tee -a $log
     return 1
  else
     if [ $dbg -eq 1 ]; then
        echo "checksums match"
     fi
  fi
  echo "`date` Applying $tarball $param" |tee -a $log
  tar -xzf $tarball 
  cd updates
  if [ "$param" = "bg" ]; then
     # use background if from Toolbox via cmc_command
     ./security_updates.sh &
     ps -ef |grep security_updates |grep -v grep 
     rv=$?
  else
     # normal useage (fg or auto) which cleans up itself afterward
     ./security_updates.sh 
     rv=$?
     cd $upddir
     rm -rf security_updates.tar.md5* security_updates.tar.gz* updates 
  fi
  cd $mydir
  cat $vardir/security_version.txt >>$log 
  if [ "$param" = "bg" ]; then
     echo "launched"
  else
     echo "done"
  fi
  if [ "$param" = "auto" ]; then
	tmpa=/tmp/security_auto.tmp
	newver=`rpm -q unitrends-security |cut -f1-4 -d'.'`
	newmsg="Applied security update $newver"
	$BPDIR/bin/send_notification.php 138 "Security Update" "$newmsg."  >$tmpa 2>&1
	echo "Send alert, ret=$?: $newmsg" |tee -a $log
	rm -f $tmpa
  fi
  return $rv
}

# main logic
export PATH=/sbin:/usr/sbin:/bin:/usr/bin:/usr/local/bin:/usr/bp/bin:/usr/bp/lib/shlib:/root/bin:/usr/local/pgsql/bin
export HOME=/root
func=$1
parm=$2
ret=0
case $func in
"available")
	checkAvailableUpdate
	ret=$?
	;;
"apply")
	applySecurityUpdate $parm
	ret=$?
	;;
*)
	echo "Usage: $0 <available|apply>"
	;;
esac
exit $ret
